"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_to_regexp_1 = require("path-to-regexp");
class Layer {
    constructor() {
        this._path = "/";
        this._opts = {};
        switch (arguments.length) {
            case 0:
                throw new Error("No arguments passed to layer!");
            case 1:
                if (typeof arguments[0] !== "function")
                    throw new TypeError("Middleware must be a function!");
                this._handler = arguments[0];
                break;
            case 2:
                if (typeof arguments[0] !== "string")
                    throw new TypeError("Path must be a string!");
                if (typeof arguments[1] !== "function")
                    throw new TypeError("Middleware must be a function!");
                this._handler = arguments[1];
                this._path = arguments[0];
                break;
            default:
                if (typeof arguments[0] !== "string")
                    throw new TypeError("Path must be a string!");
                if (typeof arguments[1] !== "object")
                    throw new TypeError("Options must be an object!");
                if (typeof arguments[2] !== "function")
                    throw new TypeError("Middleware must be a function!");
                this._handler = arguments[2];
                this._path = arguments[0];
                this._opts = arguments[1];
                break;
        }
        if (this._path === "*") {
            this._shortcut = true;
            this._path += "path";
        }
        else if (this._path === "/") {
            this._shortcut = this._opts.end === false;
        }
        else {
            this._shortcut = false;
        }
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(this._path, this._opts);
        this._regex = regexp;
        this._keys = keys;
    }
    async handle(context) {
        try {
            const match = this.match(context);
            if (match !== null) {
                await this._handler(context);
                context.query = match;
            }
        }
        catch (e) {
            throw e;
        }
    }
    match(context) {
        if (this._shortcut) {
            return "";
        }
        const match = context.query.match(this._regex);
        if (match == null)
            return null;
        for (let index = 1; index < match.length; index++) {
            context.params.set(this._keys[index - 1].name, decodeURIComponent(match[index]));
        }
        const path = context.query;
        context.query = context.query.replace(match[0], "");
        return path;
    }
    get path() {
        return this._path;
    }
    set path(value) {
        this._path = value;
        this._shortcut = false;
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(value, this._opts);
        this._keys = keys;
        this._regex = regexp;
    }
    update(value) {
        this._opts = value;
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(this._path, value);
        this._keys = keys;
        this._regex = regexp;
    }
}
exports.default = Layer;
